package cz.jzitnik.tui;

import lombok.extern.slf4j.Slf4j;

import javax.sound.sampled.*;
import java.io.IOException;

@Slf4j
public class SoundPlayer {
    public static void playSound(String filePath, int backendVolume, int masterVolume)
            throws LineUnavailableException, IOException, UnsupportedAudioFileException {
        if (!filePath.endsWith(".ogg") || masterVolume == 0) {
            return; // No sound if master volume is 0
        }

        log.info("Loading resource: {}", "sounds/" + filePath);
        var file = SoundPlayer.class.getClassLoader().getResourceAsStream("sounds/" + filePath);
        AudioInputStream audioStream = AudioSystem.getAudioInputStream(file);
        AudioFormat baseFormat = audioStream.getFormat();

        AudioFormat targetFormat = new AudioFormat(
                AudioFormat.Encoding.PCM_SIGNED,
                baseFormat.getSampleRate(),
                16,
                baseFormat.getChannels(),
                baseFormat.getChannels() * 2,
                baseFormat.getSampleRate(),
                false
        );

        AudioInputStream dataIn = AudioSystem.getAudioInputStream(targetFormat, audioStream);

        byte[] buffer = new byte[8192];
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, targetFormat);

        try (SourceDataLine line = (SourceDataLine) AudioSystem.getLine(info)) {
            if (line != null) {
                line.open(targetFormat);
                line.start();

                float finalVolume = (backendVolume / 100.0f) * (masterVolume / 100.0f);
                log.info("Applying volume: {} (backend: {}, master: {})", finalVolume, backendVolume, masterVolume);

                int bytesRead;
                while ((bytesRead = dataIn.read(buffer, 0, buffer.length)) != -1) {
                    applyVolume(buffer, bytesRead, finalVolume);
                    line.write(buffer, 0, bytesRead);
                }

                line.drain();
            }
        }

        dataIn.close();
        audioStream.close();
    }

    private static void applyVolume(byte[] buffer, int bytesRead, float volume) {
        for (int i = 0; i < bytesRead; i += 2) { // 16-bit PCM samples are 2 bytes each
            int sample = (buffer[i] & 0xFF) | (buffer[i + 1] << 8);
            sample = (int) (sample * volume);
            buffer[i] = (byte) (sample & 0xFF);
            buffer[i + 1] = (byte) ((sample >> 8) & 0xFF);
        }
    }
}
